#!/usr/bin/env python3
# -*- coding=utf-8 -*-

from typing import List, Tuple, Dict

data: List[Tuple[Tuple, Dict]] = [
    (("W", "words"), {}),
    (("S", "sentences", "ROOT"), {}),
    (("VP1", "regular verb phrases", "VP > S|SINV|SQ"), {}),
    (
        (
            "VP2",
            "verb phrases in inverted yes/no questions or in wh-questions",
            "MD|VBZ|VBP|VBD > (SQ !< VP)",
        ),
        {},
    ),
    (
        (
            "C1",
            "regular clauses",
            (
                "S|SINV|SQ "
                "[> ROOT <, (VP <# VB) | <# MD|VBZ|VBP|VBD | < "
                "(VP [<# MD|VBP|VBZ|VBD | < CC < (VP <# MD|VBP|VBZ|VBD)])]"
            ),
        ),
        {},
    ),
    (
        (
            "C2",
            "fragment clauses",
            (
                "FRAG > ROOT !<< "
                "(S|SINV|SQ [> ROOT <, (VP <# VB) | <# MD|VBZ|VBP|VBD | < "
                "(VP [<# MD|VBP|VBZ|VBD | < CC < "
                "(VP <# MD|VBP|VBZ|VBD)])])"
            ),
        ),
        {},
    ),
    (
        (
            "T1",
            "regular T-units",
            "S|SBARQ|SINV|SQ > ROOT | [$-- S|SBARQ|SINV|SQ !>> SBAR|VP]",
        ),
        {},
    ),
    (
        (
            "T2",
            "fragment T-units",
            "FRAG > ROOT !<< (S|SBARQ|SINV|SQ > ROOT | [$-- S|SBARQ|SINV|SQ !>> SBAR|VP])",
        ),
        {},
    ),
    (
        (
            "CN1",
            "complex nominals, type 1",
            "NP !> NP [<< JJ|POS|PP|S|VBG | << (NP $++ NP !$+ CC)]",
        ),
        {},
    ),
    (
        (
            "CN2",
            "complex nominals, type 2",
            "SBAR [<# WHNP | <# (IN < That|that|For|for) | <, S] & [$+ VP | > VP]",
        ),
        {},
    ),
    (("CN3", "complex nominals, type 3", "S < (VP <# VBG|TO) $+ VP"), {}),
    (
        (
            "DC",
            "dependent clauses",
            (
                "SBAR < "
                "(S|SINV|SQ "
                "[> ROOT <, (VP <# VB) | <# MD|VBZ|VBP|VBD | < "
                "(VP [<# MD|VBP|VBZ|VBD | < CC < (VP <# MD|VBP|VBZ|VBD)])])"
            ),
        ),
        {},
    ),
    (
        (
            "CT",
            "complex T-units",
            (
                "S|SBARQ|SINV|SQ [> ROOT | [$-- S|SBARQ|SINV|SQ !>> SBAR|VP]] << "
                "(SBAR < (S|SINV|SQ "
                "[> ROOT <, (VP <# VB) | <# MD|VBZ|VBP|VBD | < "
                "(VP [<# MD|VBP|VBZ|VBD | < CC < "
                "(VP <# MD|VBP|VBZ|VBD)])]))"
            ),
        ),
        {},
    ),
    (("CP", "coordinate phrases", "ADJP|ADVP|NP|VP < CC"), {}),
    (("VP", "verb phrases"), {"requirements": ["VP1", "VP2"]}),
    (("C", "clauses"), {"requirements": ["C1", "C2"]}),
    (("T", "T-units"), {"requirements": ["T1", "T2"]}),
    (("CN", "complex nominals"), {"requirements": ["CN1", "CN2", "CN3"]}),
    (("MLS", "mean length of sentence"), {"requirements": ["W", "S"]}),
    (("MLT", "mean length of T-unit"), {"requirements": ["W", "T1", "T2"]}),
    (("MLC", "mean length of clause"), {"requirements": ["W", "C1"]}),
    (("C/S", "clauses per sentence"), {"requirements": ["C1", "S"]}),
    (("VP/T", "verb phrases per T-unit"), {"requirements": ["VP1", "T1", "T2"]}),
    (("C/T", "clauses per T-unit"), {"requirements": ["C1", "T1", "T2"]}),
    (("DC/C", "dependent clauses per clause"), {"requirements": ["DC", "C1"]}),
    (("DC/T", "dependent clauses per T-unit"), {"requirements": ["DC", "T1", "T2"]}),
    (("T/S", "T-units per sentence"), {"requirements": ["T1", "T2", "S"]}),
    (("CT/T", "complex T-unit ratio"), {"requirements": ["CT", "T1", "T2"]}),
    (("CP/T", "coordinate phrases per T-unit"), {"requirements": ["CP", "T1", "T2"]}),
    (("CP/C", "coordinate phrases per clause"), {"requirements": ["CP", "C1"]}),
    (
        ("CN/T", "complex nominals per T-unit"),
        {"requirements": ["CN1", "CN2", "CN3", "T1", "T2"]},
    ),
    (("CN/C", "complex nominals per clause"), {"requirements": ["CN1", "CN2", "CN3", "C1"]}),
]



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

try:
    from xml.etree.cElementTree import XML, fromstring
except ImportError:
    from xml.etree.ElementTree import XML, fromstring
import logging
import os
import sys
from typing import ByteString, Optional, Union
import zipfile

from charset_normalizer import detect

from .util import SCAProcedureResult


class SCAIO:
    DOCX_NAMESPACE = "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}"
    DOCX_PARA = DOCX_NAMESPACE + "p"
    DOCX_TEXT = DOCX_NAMESPACE + "t"

    ODT_NAMESPACE = ".//{urn:oasis:names:tc:opendocument:xmlns:text:1.0}"
    ODT_PARA = ODT_NAMESPACE + "p"

    def __init__(self):
        self.ext_read_map = {
            ".txt": self.read_txt,
            ".docx": self.read_docx,
            ".odt": self.read_odt,
        }
        self.previous_encoding = "utf-8"

    def read_docx(self, path: str) -> str:
        """
        Take the path of a docx file as argument, return the text in unicode.
        This approach does not extract text from headers and footers.

        https://etienned.github.io/posts/extract-text-from-word-docx-simply/
        """
        with zipfile.ZipFile(path) as zip_file:
            xml_content = zip_file.read("word/document.xml")
        tree = XML(xml_content)

        paragraphs = []
        for paragraph in tree.iter(self.DOCX_PARA):
            text = "".join(node.text for node in paragraph.iter(self.DOCX_TEXT) if node.text)
            paragraphs.append(text)
        return "\n".join(paragraphs)

    def read_odt(self, path: str) -> str:
        with zipfile.ZipFile(path) as zip_file:
            xml_content = zip_file.read("content.xml")
        root = fromstring(xml_content)
        paragraphs = root.findall(self.ODT_PARA)
        return "\n".join("".join(node.itertext()) for node in paragraphs)

    def _read_txt(
        self, path: str, mode: str, encoding: Optional[str] = None
    ) -> Union[str, ByteString]:
        try:
            with open(path, mode=mode, encoding=encoding) as f:
                content = f.read()
        # input file existence has already been checked before running (main.py: verified_ifile_list),
        # here check it again in case users remove input files during runtime
        except FileNotFoundError:
            logging.critical(f"{path} does not exist.")
            sys.exit(1)
        else:
            return content

    def read_txt(self, path: str, is_guess_encoding=True) -> str:
        if not is_guess_encoding:
            content = self._read_txt(path, "r", "utf-8")
        else:
            try:
                logging.info(
                    f"Attempting to read {path} with {self.previous_encoding} encoding..."
                )
                content = self._read_txt(path, "r", self.previous_encoding)  # type:ignore
            except ValueError:
                logging.info(f"Attempt failed. Reading {path} in binary mode...")
                bytes_ = self._read_txt(path, "rb")
                logging.info("Guessing the encoding of the byte string...")
                encoding = detect(bytes_)["encoding"]  # type:ignore

                if encoding is not None:
                    logging.info(f"Decoding the byte string with {encoding} encoding...")
                    content = bytes_.decode(encoding=encoding)  # type:ignore
                    self.previous_encoding = encoding  # type:ignore
                else:
                    logging.critical(f"{path} is of unsupported file type.")
                    sys.exit(1)
        return content  # type:ignore

    def read_file(self, path: str) -> str:
        _, ext = os.path.splitext(path)
        if ext not in self.ext_read_map:
            ext = (  # assume files with other extensions as .txt files; throw an error from within read_txt() if not so
                ".txt"
            )
        return self.ext_read_map[ext](path)  # type:ignore

    @classmethod
    def try_write(cls, filename: str, content: Optional[str]) -> SCAProcedureResult:
        if not os.path.exists(filename):
            return True, None
        try:
            with open(filename, "w", encoding="utf-8") as f:
                if content is not None:
                    f.write(content)
                return True, None
        except PermissionError:
            return (
                False,
                (
                    f"PermissionError: can not write to {filename}, because it is already"
                    f" in use by another process.\n\n1. Ensure that {filename} is closed,"
                    " or \n2. Specify another output filename through the `-o` option,"
                    f" e.g. nsca input.txt -o {filename.replace('.csv', '-2.csv')}"
                ),
            )



from collections import OrderedDict
from typing import Dict, List, Optional, Set, Union

from .structure_data import data


class Structure:
    def __init__(
        self,
        name: str,
        desc: str,
        pattern: str = "",
        *,
        requirements: Optional[list] = None,
    ) -> None:
        """
        :param name: name of the structure
        :param desc: description of the structure
        :param pattern: Tregex pattern
        :param requirements: a list of structure names that current instance of Structure requires. Note that the elements come from the initial StructureCounter.structures_to_query, thus CN_T requires ["CN1", "CN2", "CN3", "T1", "T2"] instead of ["CN", "T"]
        """
        self.name = name
        self.desc = desc
        self.pattern = pattern
        if requirements is None:
            self.requirements = []
        else:
            self.requirements = requirements
        self.freq: Union[float, int] = 0
        self.matches: list = []

    def __repr__(self) -> str:  # pragma: no cover
        return (
            f"name: {self.name} ({self.desc})\nrequirements: {self.requirements}\npattern:"
            f" {self.pattern}\nmatches: {self.matches}\nfrequency: {self.freq}"
        )


class StructureCounter:
    def __init__(self, ifile="", selected_measures: Optional[Set[str]] = None) -> None:
        self.ifile = ifile
        if selected_measures is None:
            self.selected_measures = set()
        else:
            self.selected_measures = selected_measures

        self.structures: Dict[str, Structure] = {}
        for args, kwargs in data:
            self.structures[args[0]] = Structure(*args, **kwargs)

        self.structures_to_query: List[Structure] = [
            self.structures[key]
            for key in (
                "W",
                "S",
                "VP1",
                "VP2",
                "C1",
                "C2",
                "T1",
                "T2",
                "CN1",
                "CN2",
                "CN3",
                "DC",
                "CT",
                "CP",
            )
        ]

        self.structures_to_report: List[Structure] = [
            self.structures[key]
            for key in (
                "W",
                "S",
                "VP",
                "C",
                "T",
                "DC",
                "CT",
                "CP",
                "CN",
                "MLS",
                "MLT",
                "MLC",
                "C/S",
                "VP/T",
                "C/T",
                "DC/C",
                "DC/T",
                "T/S",
                "CT/T",
                "CP/T",
                "CP/C",
                "CN/T",
                "CN/C",
            )
        ]

        self.parse_selected_measures()
        self.fields = "Filename," + ",".join((s.name for s in self.structures_to_report))

    def parse_selected_measures(self):
        if len(self.selected_measures) > 0:
            self.structures_to_report = list(
                filter(lambda s: s.name in self.selected_measures, self.structures_to_report)
            )
            selected_measures_extended = self.selected_measures.copy()
            for structure in self.structures_to_report:
                for name in structure.requirements:
                    if name not in self.selected_measures:
                        selected_measures_extended.add(name)
            self.structures_to_query = [
                structure
                for structure in self.structures_to_query
                if structure.name in selected_measures_extended
            ]

    def update_freqs(self) -> None:
        """
        Update frequencies of complex nominals, clauses, verb phrases, and T-units
        """
        for s_name in ("VP", "C", "T", "CN"):
            self.structures[s_name].freq = sum(
                self.structures[requirement_name].freq
                for requirement_name in self.structures[s_name].requirements
            )

    def compute_14_indicies(self) -> None:
        """
        Compute the 14 syntactic complexity indices
        """
        for s_name, dividend, divisor in (
            ("MLS", "W", "S"),
            ("MLT", "W", "T"),
            ("MLC", "W", "C1"),
            ("C/S", "C1", "S"),
            ("VP/T", "VP1", "T"),
            ("C/T", "C1", "T"),
            ("DC/C", "DC", "C1"),
            ("DC/T", "DC", "T"),
            ("T/S", "T", "S"),
            ("CT/T", "CT", "T"),
            ("CP/T", "CP", "T"),
            ("CP/C", "CP", "C1"),
            ("CN/T", "CN", "T"),
            ("CN/C", "CN", "C1"),
        ):
            divident_freq, divisor_freq = (
                self.structures[dividend].freq,
                self.structures[divisor].freq,
            )
            self.structures[s_name].freq = (
                round(divident_freq / divisor_freq, 4) if divisor_freq else 0
            )

    def set_freq(self, structure_name: str, freq: int) -> None:
        if structure_name not in self.structures:
            raise ValueError(f"{structure_name} not counted")
        elif not isinstance(freq, int) or freq < 0:
            raise ValueError("freq should be a non-negative integer")
        else:
            self.structures[structure_name].freq = freq

    def set_matches(self, structure_name: str, matches: list) -> None:
        if structure_name not in self.structures:
            raise ValueError(f"{structure_name} not counted")
        elif not isinstance(matches, list):
            raise ValueError("matches should be a list object")
        else:
            self.structures[structure_name].matches = matches

    def get_freq(self, structure_name: str) -> Union[float, int]:
        if structure_name not in self.structures:
            raise ValueError(f"{structure_name} not counted")
        return self.structures[structure_name].freq

    def get_all_freqs(self) -> dict:
        self.update_freqs()
        self.compute_14_indicies()
        freq_dict = OrderedDict({"Filename": self.ifile})
        for structure in self.structures_to_report:
            freq_dict[structure.name] = str(structure.freq)
        return freq_dict

    def __add__(self, other: "StructureCounter") -> "StructureCounter":
        new_ifile = self.ifile + "+" + other.ifile if self.ifile else other.ifile
        new_selected_measures = self.selected_measures | other.selected_measures
        new = StructureCounter(new_ifile, selected_measures=new_selected_measures)
        for s in new.structures_to_query:
            s_name = s.name
            freq = self.get_freq(s_name) + other.get_freq(s_name)
            new.set_freq(s_name, freq)  # type: ignore
        return new



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

import os
from .cmdline_tmpl import BaseTmpl
from neosca.util_env import getenv


class TestUtilEnv(BaseTmpl):
    def test_getenv(self):
        self.assertIsNone(getenv("NON_EXISTING_VAR"))
        os.environ["NON_EXISTING_DIR"] = "non_existing_directory"
        self.assertIsNone(getenv("NON_EXISTING_DIR"))
        os.environ["USER_HOME"] = os.path.expanduser("~")
        self.assertIsNotNone(getenv("USER_HOME"))



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

import logging
import os
import re
import sys
from typing import Optional, Tuple

import jpype
from jpype import JClass

from .structure_counter import StructureCounter


class StanfordTregex:
    def __init__(
        self,
        classpaths: Optional[list] = None,
        max_memory: str = "3072m",
    ) -> None:
        self.classpaths = classpaths if classpaths is not None else []
        self.max_memory = max_memory
        self.TREGEX_PATTERN = "edu.stanford.nlp.trees.tregex.TregexPattern"
        self.STRING_READER = "java.io.StringReader"
        self.PENN_TREE_READER = "edu.stanford.nlp.trees.PennTreeReader"
        self.init_tregex()

    def init_tregex(self):
        if not jpype.isJVMStarted():  # pragma: no cover
            logging.info("[Tregex] starting JVM...")
            # Note that isJVMStarted may be renamed to isJVMRunning in the future.
            # In jpype's _core.py:
            # > TODO This method is horribly named.  It should be named isJVMRunning as
            # > isJVMStarted would seem to imply that the JVM was started at some
            # > point without regard to whether it has been shutdown.
            jpype.startJVM(f"-Xmx{self.max_memory}", classpath=self.classpaths)

        self.TregexPattern = jpype.JClass(self.TREGEX_PATTERN)
        self.StringReader = JClass(self.STRING_READER)
        self.PennTreeReader = JClass(self.PENN_TREE_READER)
        self.patname_patobj = {}

    def query_pattern(self, patname: str, pattern: str, trees: str) -> Tuple[int, list]:
        matched_subtrees = []
        if patname not in self.patname_patobj:
            tregex_pattern = self.TregexPattern.compile(pattern)
            self.patname_patobj[patname] = tregex_pattern
        else:
            tregex_pattern = self.patname_patobj[patname]

        treeReader = self.PennTreeReader(self.StringReader(trees))
        tree = treeReader.readTree()
        while tree is not None:
            matcher = tregex_pattern.matcher(tree)
            last_matching_root_node = None
            while matcher.find():
                match = matcher.getMatch()
                if last_matching_root_node is not None and last_matching_root_node == match:
                    # implement Tregex's -o option: https://github.com/stanfordnlp/CoreNLP/blob/efc66a9cf49fecba219dfaa4025315ad966285cc/src/edu/stanford/nlp/trees/tregex/TregexPattern.java#L885
                    continue
                last_matching_root_node = match
                span_string = " ".join(str(leaf.toString()) for leaf in match.getLeaves())
                # we don't use match.spanString() because the output lacks whitespace, e.g., "the media" becomes "themedia"
                penn_string = str(match.pennString().replaceAll("\r", ""))
                matched_subtrees.append(span_string + "\n" + penn_string)
            tree = treeReader.readTree()
        return len(matched_subtrees), matched_subtrees

    def query(
        self,
        counter: StructureCounter,
        trees: str,
        is_reserve_matched: bool = False,
        odir_matched: str = "",
        is_stdout: bool = False,
    ):
        for structure in counter.structures_to_query:
            logging.info(f'[Tregex] Querying "{structure.desc}"...')
            s_name = structure.name
            if structure.name == "W":
                freq = len(re.findall(r"\([A-Z]+\$? [^()—–-]+\)", trees))
                counter.set_freq(s_name, freq)
                continue
            freq, matches = self.query_pattern(structure.name, structure.pattern, trees)
            counter.set_freq(s_name, freq)
            counter.set_matches(s_name, matches)
        if is_reserve_matched:  # pragma: no cover
            self.write_match_output(counter, odir_matched, is_stdout)
        return counter

    def write_match_output(
        self, counter: StructureCounter, odir_matched: str = "", is_stdout: bool = False
    ) -> None:  # pragma: no cover
        """
        Save Tregex's match output
        """
        bn_input = os.path.basename(counter.ifile)
        bn_input_noext = os.path.splitext(bn_input)[0]
        subodir_matched = os.path.join(odir_matched, bn_input_noext).strip()
        if not is_stdout:
            os.makedirs(subodir_matched, exist_ok=True)
        for structure in counter.structures_to_query:
            if structure.matches:
                matches = "\n".join(structure.matches)
                matches_id = bn_input_noext + "-" + structure.name.replace("/", "p")
                if not is_stdout:
                    extension = ".matched"
                    fn_match_output = os.path.join(subodir_matched, matches_id + extension)
                    with open(fn_match_output, "w", encoding="utf-8") as f:
                        f.write(matches)
                else:
                    sys.stdout.write(matches_id + "\n")
                    sys.stdout.write(matches)



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

from .base_tmpl import BaseTmpl
from .base_tmpl import text
from .base_tmpl import tree as tree_expected
from .base_tmpl import classpaths
from neosca.parser import StanfordParser


class TestStanfordParser(BaseTmpl):
    def setUp(self):
        self.parser = StanfordParser(classpaths=classpaths)
        return super().setUp()

    def test_parse(self):
        tree = self.parser.parse(text)
        self.assertEqual(tree_expected, tree)
        text_never_false = """There was no possibility

of taking a walk that day."""
        text_never_true = """There was no possibility

of taking a walk that day ."""
        tree_correct = """(ROOT
  (S
    (NP (EX There))
    (VP (VBD was)
      (NP
        (NP (DT no) (NN possibility))
        (PP (IN of)
          (S
            (VP (VBG taking)
              (NP (DT a) (NN walk))
              (NP (DT that) (NN day)))))))
    (. .)))
"""
        self.assertEqual(
            tree_correct,
            self.parser.parse(text_never_false, newline_break="never", is_pretokenized=False),
        )
        self.assertEqual(
            tree_correct,
            self.parser.parse(text_never_true, newline_break="never", is_pretokenized=True),
        )
        self.assertNotEqual(
            tree_correct,
            self.parser.parse(text_never_false, newline_break="never", is_pretokenized=True),
        )
        self.assertNotEqual(
            tree_correct, self.parser.parse(text_never_false, newline_break="always")
        )
        self.assertNotEqual(
            tree_correct, self.parser.parse(text_never_false, newline_break="two")
        )
        text_always_false = """CHAPTER I
There was no possibility of taking a walk that day."""
        text_always_true = """CHAPTER I
There was no possibility of taking a walk that day ."""
        tree_correct = """(ROOT
  (S
    (VP (VB CHAPTER)
      (NP (PRP I)))))

(ROOT
  (S
    (NP (EX There))
    (VP (VBD was)
      (NP
        (NP (DT no) (NN possibility))
        (PP (IN of)
          (S
            (VP (VBG taking)
              (NP (DT a) (NN walk))
              (NP (DT that) (NN day)))))))
    (. .)))
"""
        self.assertEqual(
            tree_correct,
            self.parser.parse(text_always_false, newline_break="always", is_pretokenized=False),
        )
        self.assertEqual(
            tree_correct,
            self.parser.parse(text_always_true, newline_break="always", is_pretokenized=True),
        )
        self.assertNotEqual(
            tree_correct,
            self.parser.parse(text_always_false, newline_break="always", is_pretokenized=True),
        )
        self.assertNotEqual(
            tree_correct, self.parser.parse(text_always_false, newline_break="never")
        )
        self.assertNotEqual(
            tree_correct, self.parser.parse(text_always_false, newline_break="two")
        )
        text_two_false = """CHAPTER I

There was no possibility
of taking a walk that day."""
        text_two_true = """CHAPTER I

There was no possibility
of taking a walk that day ."""
        tree_correct = """(ROOT
  (S
    (VP (VB CHAPTER)
      (NP (PRP I)))))

(ROOT
  (S
    (NP (EX There))
    (VP (VBD was)
      (NP
        (NP (DT no) (NN possibility))
        (PP (IN of)
          (S
            (VP (VBG taking)
              (NP (DT a) (NN walk))
              (NP (DT that) (NN day)))))))
    (. .)))
"""
        self.assertEqual(
            tree_correct,
            self.parser.parse(text_two_false, newline_break="two", is_pretokenized=False),
        )
        self.assertEqual(
            tree_correct,
            self.parser.parse(text_two_true, newline_break="two", is_pretokenized=True),
        )
        self.assertNotEqual(
            tree_correct,
            self.parser.parse(text_two_false, newline_break="two", is_pretokenized=True),
        )
        self.assertNotEqual(
            tree_correct, self.parser.parse(text_two_false, newline_break="always")
        )
        self.assertNotEqual(
            tree_correct, self.parser.parse(text_two_false, newline_break="never")
        )

    def test_is_long(self):
        max_length = 100
        length_1 = 99
        length_2 = 100
        length_3 = 101
        self.assertFalse(self.parser._is_long(length_1, max_length))
        self.assertFalse(self.parser._is_long(length_2, max_length))
        self.assertTrue(self.parser._is_long(length_3, max_length))
        self.assertFalse(self.parser._is_long(length_3))



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

from neosca.querier import StanfordTregex
from neosca.structure_counter import StructureCounter

from .base_tmpl import BaseTmpl
from .base_tmpl import classpaths
from .base_tmpl import tree


class TestStanfordTregex(BaseTmpl):
    def setUp(self):
        self.counter = StructureCounter()
        self.tregex = StanfordTregex(classpaths=classpaths)
        return super().setUp()

    def test_query(self):
        counter = self.tregex.query(self.counter, tree)
        counter.update_freqs()
        self.assertEqual(1, counter.structures["S"].freq)
        self.assertEqual(2, counter.structures["VP"].freq)
        self.assertEqual(1, counter.structures["C"].freq)
        self.assertEqual(1, counter.structures["T"].freq)
        self.assertEqual(0, counter.structures["DC"].freq)
        self.assertEqual(0, counter.structures["CT"].freq)
        self.assertEqual(0, counter.structures["CP"].freq)
        self.assertEqual(1, counter.structures["CN"].freq)



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

from typing import Dict

from neosca.structure_counter import Structure
from neosca.structure_counter import StructureCounter

from .base_tmpl import BaseTmpl


class TestStructure(BaseTmpl):
    def setUp(self):
        self.structures: Dict[str, Structure] = {}
        self.structures["W"] = Structure("W", "words")
        self.structures["S"] = Structure("S", "sentences")
        return super().setUp()


class TestStructureCounter(BaseTmpl):
    def setUp(self):
        self.selected_measures1 = {"VP", "T", "DC/C"}
        self.selected_measures2 = {"CP", "VP/T", "CN/C"}
        self.counter1 = StructureCounter(selected_measures=self.selected_measures1)
        self.counter2 = StructureCounter(selected_measures=self.selected_measures2)
        return super().setUp()

    def test_parse_selected_measures(self):
        self.assertListEqual(
            ["VP", "T", "DC/C"], [s.name for s in self.counter1.structures_to_report]
        )
        self.assertListEqual(
            ["VP1", "VP2", "C1", "T1", "T2", "DC"],
            [s.name for s in self.counter1.structures_to_query],
        )
        self.assertListEqual(
            ["CP", "VP/T", "CN/C"], [s.name for s in self.counter2.structures_to_report]
        )
        self.assertListEqual(
            ["VP1", "C1", "T1", "T2", "CN1", "CN2", "CN3", "CP"],
            [s.name for s in self.counter2.structures_to_query],
        )

    def test_update_freqs(self):
        for s_name, freq in (
            ("VP1", 7),
            ("VP2", 1),
            ("C1", 9),
            ("C2", 1),
            ("T1", 11),
            ("T2", 1),
            ("CN1", 13),
            ("CN2", 1),
            ("CN3", 2),
        ):
            self.counter1.set_freq(s_name, freq)

        self.counter1.update_freqs()
        for s_name, freq in (
            ("VP", 8),
            ("C", 10),
            ("T", 12),
            ("CN", 16),
        ):
            self.assertEqual(self.counter1.structures[s_name].freq, freq)

    def test_compute_14_indices(self):
        for s_name, freq in (
            ("W", 1530),
            ("S", 54),
            ("VP1", 174),
            ("C1", 149),
            ("T", 83),
            ("DC", 61),
            ("CT", 52),
            ("CP", 26),
            ("CN", 181),
        ):
            self.counter1.set_freq(s_name, freq)

        self.counter1.compute_14_indicies()
        for s_name, freq in (
            ("W", 1530),
            ("S", 54),
            ("VP1", 174),
            ("C1", 149),
            ("T", 83),
            ("DC", 61),
            ("CT", 52),
            ("CP", 26),
            ("CN", 181),
            ("MLS", 28.3333),
            ("MLT", 18.4337),
            ("MLC", 10.2685),
            ("C/S", 2.7593),
            ("VP/T", 2.0964),
            ("C/T", 1.7952),
            ("DC/C", 0.4094),
            ("DC/T", 0.7349),
            ("T/S", 1.537),
            ("CT/T", 0.6265),
            ("CP/T", 0.3133),
            ("CP/C", 0.1745),
            ("CN/T", 2.1807),
            ("CN/C", 1.2148),
        ):
            self.assertEqual(self.counter1.structures[s_name].freq, freq)

    def test_get_freqs(self):
        freq_dict = self.counter1.get_all_freqs()
        self.assertTrue("Filename", freq_dict.keys() - set(self.selected_measures1))

    def test_add(self):
        for s_name, freq in (
            ("VP1", 5),
            ("VP2", 38),
            ("C1", 19),
            ("T1", 27),
            ("T2", 11),
            ("DC", 21),
        ):
            self.counter1.set_freq(s_name, freq)

        for s_name, freq in (
            ("VP1", 4),
            ("C1", 3),
            ("T1", 41),
            ("T2", 8),
            ("CN1", 40),
            ("CN2", 16),
            ("CN3", 49),
            ("CP", 34),
        ):
            self.counter2.set_freq(s_name, freq)

        counter3 = self.counter1 + self.counter2
        self.assertListEqual(
            ["VP", "T", "CP", "VP/T", "DC/C", "CN/C"],
            [s.name for s in counter3.structures_to_report],
        )
        self.assertListEqual(
            ["VP1", "VP2", "C1", "T1", "T2", "CN1", "CN2", "CN3", "DC", "CP"],
            [s.name for s in counter3.structures_to_query],
        )

        for s_name, freq in (
            ("VP1", 9),
            ("VP2", 38),
            ("C1", 22),
            ("T1", 68),
            ("T2", 19),
            ("DC", 21),
            ("CN1", 40),
            ("CN2", 16),
            ("CN3", 49),
            ("CP", 34),
        ):
            self.assertEqual(counter3.structures[s_name].freq, freq)
